@@ -1,6 +1,6 @@
 
 /*
- * $Id: event.c,v 1.4 1997/01/07 21:49:51 wessels Exp $
+ * $Id: event.c,v 1.4.2.2 1997/03/03 01:43:22 wessels Exp $
  *
  * DEBUG: section 41    Event Processing
  * AUTHOR: Henrik Nordstrom
@@ -61,6 +61,23 @@ eventAdd(const char *name, EVH func, void *arg, time_t when)
     *E = event;
 }
 
+void
+eventDelete(EVH func, void *arg)
+{
+    struct ev_entry **E;
+    struct ev_entry *event;
+    for (E = &tasks; (event = *E); E = &(*E)->next) {
+	if (event->func != func)
+	    continue;
+	if (event->arg != arg)
+	    continue;
+	*E = event->next;
+	xfree(event);
+	return;
+    }
+    debug_trap("eventDelete: event not found");
+}
+
 void
 eventRun(void)
 {
