@@ -1,5 +1,5 @@
 /*
- * $Id: redirect.c,v 1.32 1997/01/13 23:08:52 wessels Exp $
+ * $Id: redirect.c,v 1.34 1997/01/31 23:38:38 wessels Exp $
  *
  * DEBUG: section 29    Redirector
  * AUTHOR: Duane Wessels
@@ -62,6 +62,7 @@ static struct {
     int avg_svc_time;
     int queue_size;
     int use_hist[DefaultRedirectChildrenMax];
+    int rewrites[DefaultRedirectChildrenMax];
 } RedirectStats;
 
 
@@ -187,6 +188,8 @@ redirectHandleRead(int fd, redirector_t * redirector)
 	    fatal_dump("All redirectors have exited!");
 	return 0;
     }
+    if (len != 1)
+	RedirectStats.rewrites[redirector->index]++;
     redirector->offset += len;
     redirector->inbuf[redirector->offset] = '\0';
     /* reschedule */
@@ -263,8 +266,11 @@ GetFirstAvailable(void)
     redirector_t *redirect = NULL;
     for (k = 0; k < NRedirectors; k++) {
 	redirect = *(redirect_child_table + k);
-	if (!(redirect->flags & REDIRECT_FLAG_BUSY))
-	    return redirect;
+	if (BIT_TEST(redirect->flags, REDIRECT_FLAG_BUSY))
+	    continue;
+	if (!BIT_TEST(redirect->flags, REDIRECT_FLAG_ALIVE))
+	    continue;
+	return redirect;
     }
     return NULL;
 }
@@ -492,9 +498,10 @@ redirectStats(StoreEntry * sentry)
 	NRedirectors);
     storeAppendPrintf(sentry, "{use histogram:}\n");
     for (k = 0; k < NRedirectors; k++) {
-	storeAppendPrintf(sentry, "{    redirector #%d: %d}\n",
+	storeAppendPrintf(sentry, "{    redirector #%d: %d (%d rewrites)}\n",
 	    k + 1,
-	    RedirectStats.use_hist[k]);
+	    RedirectStats.use_hist[k],
+	    RedirectStats.rewrites[k]);
     }
     storeAppendPrintf(sentry, close_bracket);
 }
